// ..................................................................
//
//                          mdURLLabel
//
//               Copyright 1997 by Martin Djerns
//
// ..................................................................
// Initial Date : 25 April 1997 - MD
// Version 1.0  : 26 April 1997 - MD
// ..................................................................

unit mdURLLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ShellAPI, Registry;

Const
  crURLCursor = 8888;
  defWebPage = 'www.homepage.my';

type
  TmdLabelType = (Auto, Passive, Link);
  TmdURLLabel = class(TCustomLabel)
  private
    { Private declarations }
    FLinkFont : TFont;
    FPassiveFont : TFont;
    FURLCursor : TCursor;
    FWebPage : String;
    FShowHTTP : Boolean;
    FLabelType : TmdLabelType;
    FNavigator : Boolean;
    Procedure SetLinkFont(Value : TFont);
    Procedure SetPassiveFont(Value : TFont);
    Procedure SetWebPage(Value : String);
    Procedure SetShowHTTP(Value : Boolean);
    Procedure SetLabelType(Value : TmdLabelType);
  protected
    { Protected declarations }
    Procedure SetAFont(AFont, AValue : TFont);
    Procedure CheckNavigator;
    Procedure SetViewFont;
    Procedure SetTheCaption;
    Procedure Click; Override;
  public
    { Public declarations }
    Constructor Create(AOwner : TComponent); Override;
    Destructor Destroy; Override;
    Property Browser : Boolean Read FNavigator;
  published
    { Published declarations - Inherited }
    property ShowHint;
    property Transparent;
    property Color;
    property Align;
    property Alignment;
    property AutoSize;
    property Enabled;
    property ParentShowHint;
    property PopupMenu;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    { Published declarations }
    Property LinkFont : TFont Read FLinkFont Write SetLinkFont;
    Property PassiveFont : TFont Read FPassiveFont Write SetPassiveFont;

    Property WebPage : String Read FWebPage Write SetWebPage;
    Property ShowHTTP : Boolean Read FShowHTTP Write SetShowHTTP;

    Property LabelType : TmdLabelType Read FLabelType Write SetLabelType;
  end;

procedure Register;
Function GetProgramPathFromExt(Const Ext : String) : String;

implementation

{$R mdURLLabel.res}  // URL Cursor

// Pick a program from the registry associated with a extension
Function GetProgramPathFromExt(Const Ext : String) : String;
Var
  S : String;
Begin
  Result := '';
  With TRegistry.Create do
  Try
    RootKey := HKEY_CLASSES_ROOT;

    If OpenKey('\'+Ext,False) Then
    Begin
      S := ReadString('');
      If S <> '' Then
      Begin
        If OpenKey('\'+S+'\shell\open\command',False) Then
        Begin
          S := ReadString('');
          If S <> '' Then
          Begin
            Result := S;
            Exit;
          end;
        end;
      end
      else
      Begin
        If OpenKey('\'+Ext+'\shell\open\command',False) Then
        Begin
          S := ReadString('');
          If S <> '' Then
          Begin
            Result := S;
            Exit;
          end;
        end;
      end;
    end;
  Finally
    Free;
  end;
end;

Constructor TmdURLLabel.Create(AOwner : TComponent);
Begin
  Inherited Create(AOwner);
  Screen.Cursors[crURLCursor] := LoadCursor(HInstance,PChar(8888));
  CheckNavigator;

  // Default link font is normal font but
  //   in blue, and underlined
  FLinkFont := TFont.Create;
  FLinkFont.Assign(Font);
  FLinkFont.Color := clBlue;
  FLinkFont.Style := FLinkFont.Style + [fsUnderline];

  // Passive is the normal font
  FPassiveFont := TFont.Create;
  FPassiveFont.Assign(Font);

  // Set web page, and update the caption
  SetWebPage(defWebPage);

  // Set the font used for view
  SetViewFont;
  // don't show accelerator char (underline a char)
  ShowAccelChar := False;
end;

Destructor TmdURLLabel.Destroy;
Begin
  FLinkFont.Free;
  FPassiveFont.Free;
  Inherited Destroy;
end;

Procedure TmdURLLabel.SetAFont(AFont, AValue : TFont);
Begin
  If AFont <> NIL Then
    AFont.Assign(AValue);
end;

Procedure TmdURLLabel.SetLinkFont(Value : TFont);
Begin
  SetAFont(FLinkFont,Value);
end;

Procedure TmdURLLabel.SetPassiveFont(Value : TFont);
Begin
  SetAFont(FPassiveFont,Value);
end;

Procedure TmdURLLabel.SetViewFont;
Begin
  // If the label should look like a HTML link
  If (FLabelType = Link) OR
     (FNavigator AND (FLabelType = Auto)) Then
  Begin
    Font := LinkFont;
    If NOT (csDesigning IN ComponentState) Then
      Cursor := crURLCursor;
  end
  else  // If the label should look like a normal label
  Begin
    Font := PassiveFont;
    If NOT (csDesigning IN ComponentState) Then
      Cursor := crDefault;
  end;
end;

Procedure TmdURLLabel.SetWebPage(Value : String);
Begin
  FWebPage := Value;
  SetTheCaption;  // update the caption
end;

Procedure TmdURLLabel.SetShowHTTP(Value : Boolean);
Begin
  FShowHTTP := Value;
  SetTheCaption; // update the caption
end;

Procedure TmdURLLabel.SetLabelType(Value : TmdLabelType);
Begin
  FLabelType := Value;
  SetViewFont; // update the font (according to the new type)
end;

Procedure TmdURLLabel.CheckNavigator;
Begin
  // If the .html and the .htm extension is assigned to
  // a program
  FNavigator := (GetProgramPathFromExt('.html') <> '') AND
                (GetProgramPathFromExt('.htm') <> '');
end;

Procedure TmdURLLabel.SetTheCaption;
Begin
  If FShowHTTP Then
    Caption := 'http://' + FWebPage
  else
    Caption := FWebPage;
end;

Procedure TmdURLLabel.Click;
Var
  Param : PChar;
Begin
  Inherited Click;
  If (FLabelType = Link) OR
     (FNavigator AND (FLabelType = Auto)) Then
  Begin
    Param := StrAlloc(Length(WebPage)+8);
    Try
      StrPCopy(Param,'http://'+WebPage+#0);
      // Execute the default web browser on the
      // web page
      ShellExecute(0,
                   'open',
                   Param,
                   NIL,
                   NIL,
                   SW_SHOWNORMAL);
    Finally
      StrDispose(Param);
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('mdVCL', [TmdURLLabel]);
end;

end.
